## ----setup, include = FALSE----------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ---- warning = F, message = F-------------------------------------------
library(LWFBrook90R)
library(data.table)

## ------------------------------------------------------------------------
options.b90 <- setoptions_LWFB90()
param.b90 <- setparam_LWFB90()

## ------------------------------------------------------------------------
soil <- cbind(slb1_soil, hydpar_wessolek_mvg(tex.KA5 = slb1_soil$texture))

## ------------------------------------------------------------------------
output <- setoutput_LWFB90()
output

## ---- eval = F-----------------------------------------------------------
#  b90res <- runLWFB90(project.dir = "example_run_b90/",
#                              options.b90 = options.b90,
#                              param.b90 = param.b90,
#                              climate = slb1_meteo,
#                              soil = soil,
#                              output = output)

## ---- echo = F-----------------------------------------------------------
b90res <- LWFBrook90R:::b90res

## ------------------------------------------------------------------------
str(b90res$EVAPDAY.ASC)

## ------------------------------------------------------------------------
b90res$EVAPDAY.ASC[, dates := as.Date(paste(YR, MO,DA, sep = "-"))]

## ------------------------------------------------------------------------
b90res$SWATDAY.ASC[, dates := as.Date(paste(YR, MO,DA, sep = "-"))]
swat100cm <- b90res$SWATDAY.ASC[which(NL <= 14), list(swat100cm = sum(SWATI)),
                                by  = dates]
                                          

## ---- fig.height=5, fig.width=7, echo =F, fig.cap="Simulation results for sample data"----
par(mar=c(4.1,4.1,1.1,4.1), oma = c(1,1,1,1))
plot(b90res$EVAPDAY.ASC$dates, 
     b90res$EVAPDAY.ASC$TRAN, type ='l',
     col = "green", ylab = "tran [mm]", xlab = "")
par(new =T)
plot(swat100cm$dates,
     swat100cm$swat100cm, 
     ylim=c(100, 350), type ='l', col = "blue",
     xaxt = "n", yaxt ="n", xlab= "", ylab = "")
axis(4,pretty(c(100,350)))
mtext("swat_100cm [mm]", side = 4, line =3)
legend("bottom",inset = -0.25,
       legend = c("tran", "swat100cm"),
       col = c("green", "blue"),  lty = 1, 
      bty = "n", xpd = T,  horiz = T,  text.width = 100)

## ------------------------------------------------------------------------
output_function <- function(x) {
  # aggregate SWAT
  swat_tran <- x$SWATDAY.ASC[which(NL <= 14), 
                             list(swat100cm = sum(SWATI)),
                             by  = list(dates = as.Date(paste(YR, MO,DA, sep = "-")))]
  #add transpiration from EVAPDAY.ASC
  swat_tran$tran <- x$EVAPDAY.ASC$TRAN
  return(swat_tran)
}

## ------------------------------------------------------------------------
head(output_function(b90res))

## ------------------------------------------------------------------------
N=50
paramvar <- data.frame(maxlai = runif(N, 4,7),
                       glmax = runif(N,0.003, 0.01))

## ---- eval = F-----------------------------------------------------------
#  mrun_res <- mrunLWFB90(paramvar = paramvar,
#                         param.b90 = param.b90,
#                         cores = 3,
#                         options.b90 = options.b90, # the following args are passed to runLWFB90
#                         climate = slb1_meteo,
#                         soil = soil,
#                         output = output,
#                         rtrn.input = F, rtrn.output = F,
#                         output_fun = output_function)

## ---- eval = F-----------------------------------------------------------
#  mrun_dt <- rbindlist(lapply(mrun_res, function(x) x$output_fun[[1]]),
#                        idcol = "singlerun")

## ---- eval = F, echo = F-------------------------------------------------
#  # vignette data: store in data-raw and include later as internal data with script 'make_internal_data'
#  save(mrun_dt, b90res, file = file.path(dirname(getwd()),"data-raw/vign_dat.rda"))

## ----results='hide', fig.height=5, fig.width=7, echo =F, fig.cap = "Transpiration and  soil water storage of 50 simulations, with random variation of parameters 'maxlai' and 'glmax' "----

par(mar=c(4.1,4.1,1.1,4.1), oma = c(1,1,1,1))
LWFBrook90R:::mrun_dt[singlerun == "RunNo.1",
        plot(dates, cumsum(tran),
             type = "n", col = "green",
             ylab = "cum. Transpiration [mm]",
             ylim = c(0,700))]

LWFBrook90R:::mrun_dt[,lines(dates, cumsum(tran), col = "green"),
        by = singlerun]

LWFBrook90R:::mrun_dt[,lines(dates, swat100cm*2, col = "blue"),
        by = singlerun]
axis(4, at = seq(0,700,100), labels = seq(0,700,100)/2)
mtext("Soil water storage in 0-100 cm [mm]", side = 4, line = 3 )
legend("bottomright",
       pch = NA, col =c("blue", "green"),lty = 1, 
       legend = c("SWAT", "TRAN"),bty = "n")


## ------------------------------------------------------------------------
LAI_b90 <-  MakeSeasLAI(method = options.b90$lai.method,
                        year = 2003,
                        maxlai = param.b90$maxlai,
                        winlaifrac = param.b90$winlaifrac,
                        budburst.doy = param.b90$budburstdoy,
                        leaffall.doy = param.b90$leaffalldoy,
                        emerge.dur = param.b90$emergedur,
                        leaffall.dur = param.b90$leaffalldur)

## ------------------------------------------------------------------------
options.b90$lai.method <- "linear"
param.b90$lai.doy <- c(1,110,117,135,175,220,250,290,365)
param.b90$lai.frac <- c(0.1,0.1,0.5,0.7,1.2,1.2,1.0,0.1,0.1)
LAI_linear <-  MakeSeasLAI(method = options.b90$lai.method,
                           year = 2003,
                           maxlai = param.b90$maxlai,
                           lai.doy = param.b90$lai.doy ,
                           lai.frac = param.b90$lai.frac)

## ------------------------------------------------------------------------
options.b90$lai.method <- "Coupmodel"
param.b90$shape.budburst <- 0.5
param.b90$shape.leaffall <- 5
param.b90$shape.optdoy <- 180
LAI_coupmodel <-  MakeSeasLAI(method = options.b90$lai.method,
                              year = 2003,
                              maxlai = param.b90$maxlai,
                              budburst.doy = param.b90$budburstdoy,
                              leaffall.doy = param.b90$leaffalldoy,
                              shape.budburst = param.b90$shape.budburst,
                              shape.leaffall = param.b90$shape.leaffall,
                              shape.optdoy = param.b90$shape.optdoy)

## ---- echo = F, fig.height=5, fig.width=7, fig.cap = "Methods featured by MakeSeasLAI()"----
par(xpd = TRUE, mar = c(5.1,4.1,2.1,2.1), oma = c(1,1,1,1))

plot(LAI_b90, type = "n", xlab = "doy", ylab = "lai [m²/m²]", ylim = c(0,6))
with(param.b90, abline(v = c(budburstdoy,budburstdoy+emergedur,
                             leaffalldoy, leaffalldoy+leaffalldur), lty = 2, xpd = F))
lines(LAI_b90, col ="green",lwd = 2,)
lines(LAI_linear, col ="red",lwd = 2)
lines(LAI_coupmodel, col ="blue",lwd = 2)

with(param.b90, arrows(x0 = c(budburstdoy,leaffalldoy,shape.optdoy,budburstdoy,leaffalldoy),
                       x1 = c(budburstdoy,leaffalldoy, shape.optdoy,
                              budburstdoy+emergedur, leaffalldoy + leaffalldur),
                       y0 = c(-1.6,-1.6,3.5,2.5,2.5),y1 = c(-0.3,-0.3,4.8,2.5,2.5), 
                       length = 0.15, code = 2))
with(param.b90, arrows(x0 = c(budburstdoy,leaffalldoy),
                       x1 = c(budburstdoy+emergedur, leaffalldoy + leaffalldur),
                       y0 = c(2.5,2.5),y1 = c(2.5,2.5), 
                       length = 0.15, code = 3))

with(param.b90, text(x = c(budburstdoy, leaffalldoy, shape.optdoy), y = c(-1.9,-1.9,3.2), c("budburstdoy", "leaffalldoy", "shape.optdoy")))
with(param.b90, text(x = c(budburstdoy+0.5*emergedur,leaffalldoy+0.5*leaffalldur),
                     y = 2, c("emergedur","leaffalldur")))
with(param.b90, text(
  x = c(budburstdoy/2, (leaffalldoy - budburstdoy - emergedur)/2 + budburstdoy + emergedur),y = c(winlaifrac*maxlai+0.4,maxlai+0.4), c("winlaifrac * maxlai", "maxlai")))

legend("topleft",c("'b90'","'linear'", "'Coupmodel'"), pch = NULL, lwd =2, col = c("green", "red", "blue"), bty = "n")


## ----echo = 1:6, results = 'hide',fig.height=5, fig.width=7, fig.cap = "Options and parameters affecting interannual variation of leaf area index."----
years <- 2001:2003
param.b90$maxlai <- c(4,6,5)
param.b90$shape.optdoy <- c(210,180,240)
param.b90$shape.budburst <- c(3,1,0.3)
param.b90$budburstdoy <- c(100,135,121) 
lai_variation <- MakeSeasLAI(method = options.b90$lai.method,
                              year = years,
                              maxlai = param.b90$maxlai,
                              budburst.doy = param.b90$budburstdoy,
                              leaffall.doy = param.b90$leaffalldoy,
                              shape.budburst = param.b90$shape.budburst,
                              shape.leaffall = param.b90$shape.leaffall,
                              shape.optdoy = param.b90$shape.optdoy)
par(mar=c(4.1,4.1,1.1,4.1), oma = c(1,1,1,1))
plot(seq.Date(as.Date("2001-01-01"), as.Date("2003-12-31"), by = "day"),
     lai_variation, col = "green", ylab = "lai [m²/m²]",
                                           type ="l", xlab = "", lwd = 2)
arrows( x0 = as.Date(paste(param.b90$budburstdoy, years),format = "%j %Y"),
     y0 = -1.0, y1 = -0.3, length = 0.15, code = 2, xpd =T)
text(x = as.Date(paste(param.b90$budburstdoy, years),format = "%j %Y"),
     y = -1.3, paste("doy", param.b90$budburstdoy), xpd = T)

## ------------------------------------------------------------------------
# constant 'interpolation'
options.b90$standprop.interp <- 'constant'
param.b90$height <- c(20.2,20.8,21.3)
simyears <- 2002:2003
height_c <- approx_standprop(x.years=years,
                                 y = param.b90$height,
                                 approx.method = options.b90$standprop.interp)

# linear interpolation       
options.b90$standprop.interp <- 'linear'
param.b90$height.ini <- 19.1
height_l <- approx_standprop(x.years=years,
                             y = param.b90$height,
                             y.ini = param.b90$height.ini,
                             approx.method = options.b90$standprop.interp)

## ------------------------------------------------------------------------
options.b90$standprop.use_growthperiod <- TRUE
height_l_gp <- approx_standprop(x.years = years,
                             y = param.b90$height,
                             y.ini = param.b90$height.ini,
                             use_growthperiod = options.b90$standprop.use_growthperiod,
                             startdoy = param.b90$budburstdoy,
                             enddoy = param.b90$leaffalldoy,
                             approx.method = options.b90$standprop.interp)

## ----echo = F, fig.height=5, fig.width=7, fig.cap="Interpolated stand height derived from parameters using approx_standprop()"----
dates <- seq.Date(from = as.Date(paste0(min(years),"-01-01")),
                           to = as.Date(paste0(max(years),"-12-31")),
                           by = "day")
par(mar=c(4.1,4.1,1.1,4.1))
plot(dates, height_c,
     type = "l", lwd = 2, col = "black", 
     ylim = c(19,22), ylab = "height [m]", xlab = "", xpd = T)
lines(dates, height_l, 
      col = "blue", lwd = 2)
lines(dates, height_l_gp, 
      col = "green", lwd = 2)
legend("topleft", legend = c("approx.method = 'constant'", "approx.method = 'linear'", "approx.method = 'linear', use_growthperiod = TRUE"), 
       col  = c("black", "blue", "green"),  lwd = 2, pch = NULL,
       bty = "n")
arrows( x0 = as.Date(paste(param.b90$budburstdoy, years),format = "%j %Y"),
     y0 = c(param.b90$height.in,param.b90$height[1:2])-0.5, y1 = c(param.b90$height.in,param.b90$height[1:2])-0.1, length = 0.15, code = 2, xpd =T)
text(x = as.Date(paste(param.b90$budburstdoy, years), format = "%j %Y"),
     y = c(param.b90$height.in,param.b90$height[1:2])-0.7, paste("doy", param.b90$budburstdoy), xpd = T)
arrows( x0 = as.Date(paste(param.b90$leaffalldoy, years),format = "%j %Y"),
     y0 = param.b90$height-0.5, y1 = param.b90$height-0.1, length = 0.15, code = 2, xpd =T)
text(x = as.Date(paste(param.b90$leaffalldoy, years), format = "%j %Y"),
     y = param.b90$height-0.7, paste("doy", param.b90$leaffalldoy), xpd = T)

## ---- eval = F, messages = F---------------------------------------------
#  #Extend simulation period
#  options.b90$startdate <- as.Date("1980-01-01")
#  options.b90$enddate <- as.Date("1999-12-31")
#  
#  #set up options for table input
#  options.b90$standprop.input <- 'table'
#  param.b90$standprop.table <- slb1_standprop
#  
#  # Set up dynamic budburst and leaf fall
#  options.b90$budburst.method <- "Menzel"
#  options.b90$leaffall.method <- "vonWilpert"
#  param.b90$budburst.species <- "Fagus sylvatica"
#  
#  #run LWF-Brook90 without simulation
#  b90res.standtable <- runLWFB90(project.dir = "example_run_b90/",
#                                         options.b90 = options.b90,
#                                         param.b90 = param.b90,
#                                         climate = slb1_meteo,
#                                         soil = soil,
#                                         output = output,
#                                         run = F,
#                                         verbose = F)
#  
#  par(mar=c(4.1,4.1,1.1,4.1), oma = c(1,1,1,1))
#  with(b90res.standtable$standprop_daily,{
#    plot(dates, lai,
#         type = "l", lwd = 2, col = "green",
#         ylab = "lai, sai m²/m²", xlab = "")
#    lines(dates, sai, lwd = 2, col = "brown")
#    par(new = T)
#    plot(dates, height,
#         type = "l", lwd = 2, col = "blue",
#         ylim = c(27, 32),
#         ylab = "", xlab = "",
#         xaxt = "n", yaxt = "n")
#  })
#  axis(4, at = seq(27,32, by = 1))
#  mtext("height [m]", side = 4, line = 3)
#  legend("bottom",horiz = T, bty = "n",inset = -0.3, xpd =T,
#         legend = c("lai", "sai", "height"),
#         col  = c("green","brown", "blue"), lwd = 2, pch = NULL)

## ---- echo = F, fig.height=4, fig.width=4--------------------------------
plot(MakeRelRootDens(soilnodes = seq(0,-2, by = -0.01), 
                              method = "betamodel", 
                              beta = 0.93),
     seq(0,-2, by = -0.01), ylim = c(-2,0),
     type="l", lwd = 1.5,
     xlab = "relative root density", ylab = "soil depth [m]")
lines(MakeRelRootDens(soilnodes = seq(0,-2, by = -0.01), 
                              method = "betamodel", 
                              beta = 0.96),
     seq(0,-2, by = -0.01), 
     lty = 2, lwd = 1.5)
lines(MakeRelRootDens(soilnodes = seq(0,-2, by = -0.01), 
                              method = "betamodel", 
                              beta = 0.98),
     seq(0,-2, by = -0.01), 
     lty = 3, lwd = 1.5)
lines(MakeRelRootDens(soilnodes = seq(0,-2, by = -0.01), 
                              method = "betamodel", 
                              beta = 0.99),
     seq(0,-2, by = -0.01), 
     lty = 4, lwd = 1.5)
legend("bottomright",legend = c(expression(paste(beta, "= 0.93")),expression(paste(beta, "= 0.96")),
                                expression(paste(beta, "= 0.98")),expression(paste(beta, "= 0.99"))),
       lwd = 1.5, pch = NULL, lty = 1:4, bty = "n")

## ------------------------------------------------------------------------
param.b90$maxrootdepth <- -1.4
options.b90$root.method <- "betamodel" 
roots_beta <- MakeRelRootDens(soilnodes = slb1_soil$lower, 
                              maxrootdepth = param.b90$maxrootdepth,
                              beta = param.b90$betaroot,
                              method = options.b90$root.method)

## ------------------------------------------------------------------------
options.b90$root.method <- 'table'
param.b90$rootden.table <- data.frame(
  depth = c(-0.02, -0.15, -0.35, -0.5, -0.65,-0.9,-1.1,-1.3,-1.6),
  rootden = c(15, 35, 15, 7.5, 4, 12, 2, 2, 0))
roots_table <- MakeRelRootDens(soilnodes = slb1_soil$lower, 
                              method = options.b90$root.method, 
                              relrootden = param.b90$rootden.table$rootden,
                              rootdepths = param.b90$rootden.table$depth)

## ---- echo = 1:4, fig.height=4, fig.width=4------------------------------
options.b90$root.method <- 'linear'
roots_linear <- MakeRelRootDens(soilnodes = slb1_soil$lower, 
                              maxrootdepth = param.b90$maxrootdepth,
                              method = options.b90$root.method,
                              relrootden = 0.2)
options.b90$root.method <- 'const'
roots_constant <- MakeRelRootDens(soilnodes = slb1_soil$lower, 
                              maxrootdepth = param.b90$maxrootdepth,
                              method = options.b90$root.method,
                              relrootden = 0.2)

plot(roots_constant, slb1_soil$lower +runif(n=length(slb1_soil$lower), -0.02,0.02),
     type = 's', lwd = 1.5,ylab = "soil depth [m]",xlab = "relative root density",
     xlim = c(0,0.35), col = "red")
lines(roots_linear, slb1_soil$lower,
      type = 's', col = "blue", lwd = 1.5)
lines(roots_table/100, slb1_soil$lower+runif(n=length(slb1_soil$lower), -0.02,0.02),
      type = 's', col = "green", lwd = 1.5)
lines(roots_beta, slb1_soil$lower, type = 's', col = "brown", lwd = 1.5)

legend("bottomright", c("'betamodel'","'table'","'linear'", "'constant'"),seg.len = 1.5,
       pch = NULL, lwd =1.5, col = c("brown", "green", "blue", "red"), bty = "n")


